@@ -1,6 +1,6 @@
 # --
 # Copyright (C) 2001-2021 OTRS AG, https://otrs.com/
-# Copyright (C) 2021-2022 Znuny GmbH, https://znuny.org/
+# Copyright (C) 2021 Znuny GmbH, https://znuny.org/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
@@ -594,25 +594,16 @@ sub Run {
                 qw(ServiceID OwnerID Owner ResponsibleID Responsible PriorityID Priority QueueID Queue Subject
                 Body IsVisibleForCustomer TypeID StateID State MergeToSelection MergeTo LinkTogether
                 EmailSubject EmailBody EmailTimeUnits
-                LinkTogetherParent Unlock MergeToChecked MergeToOldestChecked)
+                LinkTogetherParent Unlock MergeToChecked MergeToOldestChecked MarkTicketsAs)
                 )
             {
                 $GetParam{$Key} = $ParamObject->GetParam( Param => $Key ) || '';
             }
 
-            for my $Key (qw(TimeUnits)) {
+            for my $Key (qw(TimeUnits Watch)) {
                 $GetParam{$Key} = $ParamObject->GetParam( Param => $Key );
             }
 
-            # get time stamp based on user time zone
-            %Time = $LayoutObject->TransformDateSelection(
-                Year   => $ParamObject->GetParam( Param => 'Year' ),
-                Month  => $ParamObject->GetParam( Param => 'Month' ),
-                Day    => $ParamObject->GetParam( Param => 'Day' ),
-                Hour   => $ParamObject->GetParam( Param => 'Hour' ),
-                Minute => $ParamObject->GetParam( Param => 'Minute' ),
-            );
-
             if ( $GetParam{'MergeToSelection'} eq 'OptionMergeTo' ) {
                 $GetParam{'MergeToChecked'} = 'checked';
             }
@@ -673,6 +664,15 @@ sub Run {
 
                 if ( $StateData{TypeName} =~ /^pending/i ) {
 
+                    # get time stamp based on user time zone
+                    %Time = $LayoutObject->TransformDateSelection(
+                        Year   => $ParamObject->GetParam( Param => 'Year' ),
+                        Month  => $ParamObject->GetParam( Param => 'Month' ),
+                        Day    => $ParamObject->GetParam( Param => 'Day' ),
+                        Hour   => $ParamObject->GetParam( Param => 'Hour' ),
+                        Minute => $ParamObject->GetParam( Param => 'Minute' ),
+                    );
+
                     # create datetime object
                     my $PendingDateTimeObject = $Kernel::OM->Create(
                         'Kernel::System::DateTime',
@@ -1237,7 +1237,7 @@ sub Run {
                     TICKETIDPARTNER:
                     for my $TicketIDPartner (@TicketIDs) {
 
-                        next TICKETIDPARTNER if $TicketID ne $TicketIDPartner;
+                        next TICKETIDPARTNER if $TicketID == $TicketIDPartner;
 
                         $LinkObject->LinkAdd(
                             SourceObject => 'Ticket',
@@ -1317,6 +1317,74 @@ sub Run {
                         );
                     }
                 }
+
+                # watch or unwatch tickets
+                if ( defined $GetParam{'Watch'} ) {
+                    if ( $GetParam{'Watch'} eq '1' ) {
+                        $Result = $TicketObject->TicketWatchSubscribe(
+                            TicketID    => $TicketID,
+                            WatchUserID => $Self->{UserID},
+                            UserID      => $Self->{UserID},
+                        );
+
+                        if ( !$Result ) {
+                            push @NonUpdatedTickets, $Ticket{TicketNumber};
+                        }
+                    }
+                    elsif ( $GetParam{'Watch'} eq '0' ) {
+                        $Result = $TicketObject->TicketWatchUnsubscribe(
+                            TicketID    => $TicketID,
+                            WatchUserID => $Self->{UserID},
+                            UserID      => $Self->{UserID},
+                        );
+
+                        if ( !$Result ) {
+                            push @NonUpdatedTickets, $Ticket{TicketNumber};
+                        }
+                    }
+                }
+
+                if ( $GetParam{'MarkTicketsAs'} eq 'Seen' || $GetParam{'MarkTicketsAs'} eq 'Unseen' ) {
+
+                    my $TicketActionFunction;
+                    my $ArticleActionFunction;
+
+                    if ( $GetParam{'MarkTicketsAs'} eq 'Seen' ) {
+                        $TicketActionFunction  = 'TicketFlagSet';
+                        $ArticleActionFunction = 'ArticleFlagSet';
+
+                    }
+                    elsif ( $GetParam{'MarkTicketsAs'} eq 'Unseen' ) {
+                        $TicketActionFunction  = 'TicketFlagDelete';
+                        $ArticleActionFunction = 'ArticleFlagDelete';
+                    }
+
+                    my @ArticleIDs = $ArticleObject->ArticleIndex(
+                        TicketID => $TicketID,
+                    );
+
+                    ARTICLEID:
+                    for my $ArticleID ( sort @ArticleIDs ) {
+
+                        # article flag
+                        my $Success = $ArticleObject->$ArticleActionFunction(
+                            TicketID  => $TicketID,
+                            ArticleID => $ArticleID,
+                            Key       => 'Seen',
+                            Value     => 1,                 # irrelevant in case of delete
+                            UserID    => $Self->{UserID},
+                        );
+                    }
+
+                    # ticket flag
+                    $TicketObject->$TicketActionFunction(
+                        TicketID => $TicketID,
+                        Key      => 'Seen',
+                        Value    => 1,                      # irrelevant in case of delete
+                        UserID   => $Self->{UserID},
+                    );
+                }
+
                 $ActionFlag = 1;
             }
             $Counter++;
@@ -1690,18 +1758,22 @@ sub _Mask {
 
     # show time accounting box
     if ( $ConfigObject->Get('Ticket::Frontend::AccountTime') ) {
+        my $IsTimeUnitsRequired = $ConfigObject->Get('Ticket::Frontend::NeedAccountedTime');
+        $LayoutObject->AddJSData(
+            Key   => 'TimeUnitsRequired',
+            Value => $IsTimeUnitsRequired || '',
+        );
 
-        $Param{TimeUnitsRequired} = (
-            $ConfigObject->Get('Ticket::Frontend::NeedAccountedTime')
-            ? 'Validate_DependingRequiredAND Validate_Depending_Subject'
-            : ''
+        my $TimeUnitsInputType = $ConfigObject->Get('Ticket::Frontend::AccountTimeType') // 'Text';
+        $LayoutObject->AddJSData(
+            Key   => 'TimeUnitsInputType',
+            Value => $TimeUnitsInputType,
         );
 
         $Param{TimeUnitsBlock} = $LayoutObject->TimeUnits(
-            ID                => 'TimeUnits',
-            Name              => 'TimeUnits',
-            TimeUnits         => $Param{TimeUnits},
-            TimeUnitsRequired => $Param{TimeUnitsRequired},
+            ID        => 'TimeUnits',
+            Name      => 'TimeUnits',
+            TimeUnits => $Param{TimeUnits},
         );
 
         $LayoutObject->Block(
@@ -1709,17 +1781,10 @@ sub _Mask {
             Data => \%Param,
         );
 
-        $Param{TimeUnitsRequired} = (
-            $ConfigObject->Get('Ticket::Frontend::NeedAccountedTime')
-            ? 'Validate_DependingRequiredAND Validate_Depending_EmailSubject'
-            : ''
-        );
-
         $Param{EmailTimeUnitsBlock} = $LayoutObject->TimeUnits(
-            ID                => 'EmailTimeUnits',
-            Name              => 'EmailTimeUnits',
-            TimeUnits         => $Param{EmailTimeUnits},
-            TimeUnitsRequired => $Param{TimeUnitsRequired},
+            ID        => 'EmailTimeUnits',
+            Name      => 'EmailTimeUnits',
+            TimeUnits => $Param{EmailTimeUnits},
         );
         $LayoutObject->Block(
             Name => 'EmailTimeUnits',
@@ -1741,6 +1806,59 @@ sub _Mask {
         Class      => 'Modernize',
     );
 
+# Ticket::WatcherGroup - Enables or disables the ticket watcher feature, to keep track of tickets without being the owner nor the responsible.
+    my @WatcherGroups = @{ $ConfigObject->Get('Ticket::WatcherGroup') // [] };
+    my $BulkWatch     = 0;
+
+    # General permission via config switch to use ticket watcher.
+    if ( $ConfigObject->Get('Ticket::Watcher') ) {
+        $BulkWatch = 1;
+    }
+
+    # Ticket watcher via group.
+    elsif (@WatcherGroups) {
+        my $GroupObject = $Kernel::OM->Get('Kernel::System::Group');
+        GROUP:
+        for my $Group (@WatcherGroups) {
+            my $HasPermission = $GroupObject->PermissionCheck(
+                UserID    => $Self->{UserID},
+                GroupName => $Group,
+                Type      => 'rw',
+            );
+            next GROUP if !$HasPermission;
+
+            $BulkWatch = 1;
+            last GROUP;
+        }
+    }
+
+    if ($BulkWatch) {
+        $Param{WatchYesNoOption} = $LayoutObject->BuildSelection(
+            Data         => $ConfigObject->Get('YesNoOptions'),
+            Name         => 'Watch',
+            PossibleNone => 1,
+            SelectedID   => $Param{Watch} // '',
+            Class        => 'Modernize',
+        );
+
+        $LayoutObject->Block(
+            Name => 'Watch',
+            Data => \%Param,
+        );
+    }
+
+    $Param{MarkTicketsAsOption} = $LayoutObject->BuildSelection(
+        Data => {
+            Seen   => 'Mark as seen',
+            Unseen => 'Mark as unseen',
+        },
+        Name         => 'MarkTicketsAs',
+        SelectedID   => $Param{MarkTicketsAs} // 0,
+        PossibleNone => 1,
+        Translation  => 1,
+        Class        => 'Modernize',
+    );
+
     # add rich text editor for note & email
     if ( $LayoutObject->{BrowserRichText} ) {
 
@@ -1807,7 +1925,9 @@ sub _Mask {
             my $DynamicFieldSet = $ParamObject->GetParam(
                 Param => 'DynamicField_' . $DynamicFieldConfig->{Name} . 'Used'
             );
-            if ($DynamicFieldSet) {
+
+            # set DynamicField_NAMEUsed checkbox to true if it is set before or if mandatory (2)
+            if ( $DynamicFieldSet || $Config->{DynamicField}->{ $DynamicFieldConfig->{Name} } == 2 ) {
                 $IsChecked = 'true';
             }
         }
@@ -1951,9 +2071,11 @@ sub _GetTypes {
 sub _GetOwners {
     my ( $Self, %Param ) = @_;
 
+    my $UserObject = $Kernel::OM->Get('Kernel::System::User');
+
     # Get all users.
-    my %AllGroupsMembers = $Kernel::OM->Get('Kernel::System::User')->UserList(
-        Type  => 'Long',
+    my %AllGroupsMembers = $UserObject->UserList(
+        Type  => 'Short',
         Valid => 1
     );
 
@@ -2010,16 +2132,28 @@ sub _GetOwners {
         UserID        => $Self->{UserID},
     );
 
-    return $TicketObject->TicketAclData() if $ACL;
+    if ($ACL) {
+        %OwnerList = $TicketObject->TicketAclData();
+    }
+
+    my %AllGroupsMembersFullnames = $UserObject->UserList(
+        Type  => 'Long',
+        Valid => 1,
+    );
+
+    @OwnerList{ keys %OwnerList } = @AllGroupsMembersFullnames{ keys %OwnerList };
+
     return %OwnerList;
 }
 
 sub _GetResponsibles {
     my ( $Self, %Param ) = @_;
 
+    my $UserObject = $Kernel::OM->Get('Kernel::System::User');
+
     # Get all users.
-    my %AllGroupsMembers = $Kernel::OM->Get('Kernel::System::User')->UserList(
-        Type  => 'Long',
+    my %AllGroupsMembers = $UserObject->UserList(
+        Type  => 'Short',
         Valid => 1
     );
 
@@ -2076,7 +2210,17 @@ sub _GetResponsibles {
         UserID        => $Self->{UserID},
     );
 
-    return $TicketObject->TicketAclData() if $ACL;
+    if ($ACL) {
+        %ResponsibleList = $TicketObject->TicketAclData();
+    }
+
+    my %AllGroupsMembersFullnames = $UserObject->UserList(
+        Type  => 'Long',
+        Valid => 1,
+    );
+
+    @ResponsibleList{ keys %ResponsibleList } = @AllGroupsMembersFullnames{ keys %ResponsibleList };
+
     return %ResponsibleList;
 }
 
